-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp - all rights reserved.
-- 
-- For more scripts and sample code, check out 
-- 	http://www.SQLSkills.com
-- For "More than Just Training," see the Industry Experts at 
-- 	http://www.SolidQualityLearning.com
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

-- Use this pseudo code to get an idea of how isolation levels affect reads
-- BEGIN TRAN
-- sql
-- Q1 = select count(*) from tname where country = 'USA'
-- sql
-- sql
-- sql
-- sql
-- Q2 = select count(*) from tname where country = 'USA'
-- sql
-- COMMIT TRAN
-- 
-- IsolatiON Level
-- 0	Read Uncommitted = NOLOCK
-- 	Q1 > Q2, Q2 > Q1 or Q1 = Q2 - no guarantee
-- 	Could even read data that has not yet been committed
-- 1	Read Committed (DEFAULT)
-- 	Q1 > Q2, Q2 > Q1 or Q1 = Q2 
-- 	guarantee only committed values are shown
-- 	(also known AS Inconsistent Analysis Problem)
-- 2	Repeatable Reads 
-- 	solves the inconsistent analysis problem
-- 	Q1 <= Q2 
-- 	halloween problem
-- 	problem is called a phantom when Q2 > Q1 
-- 	new records introduced into set
-- 3	Serializable = HOLDLOCK
-- 	Q1 = Q2
-- 

--Setting these in a query on a table by table basis
select t.title, p.pub_name
from dbo.titles AS t WITH (NOLOCK)
	inner join dbo.publishers AS p WITH (HOLDLOCK)
		ON p.pub_id = t.pub_id

-- Setting these for a session

SET TRANSACTION ISOLATION LEVEL READUNCOMMITTED -- 0
SET TRANSACTION ISOLATION LEVEL READCOMMITTED -- 1
SET TRANSACTION ISOLATION LEVEL REPEATABLEREAD -- 2
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE -- 3

-- Join hints
SELECT t.title, p.pub_name
FROM dbo.titles AS t WITH (NOLOCK)
	INNER LOOP JOIN dbo.publishers AS p WITH (HOLDLOCK)
		ON p.pub_id = t.pub_id

SELECT t.title, p.pub_name
FROM dbo.titles AS t WITH (NOLOCK)
	INNER HASH JOIN dbo.publishers AS p WITH (HOLDLOCK)
		ON p.pub_id = t.pub_id

SELECT t.title, p.pub_name
FROM dbo.titles AS t WITH (NOLOCK)
	INNER MERGE JOIN dbo.publishers AS p WITH (HOLDLOCK)
		ON p.pub_id = t.pub_id

-- Expand Views - tells use base tables (even when index ON view use base tables)
SELECT member_no AS MemberNo, 
	sum(charge_amt) AS TotalSales
FROM dbo.charge WITH (index (charge_charge_amt))
GROUP BY member_no
OPTION (EXPAND VIEWS)

-- NOEXPAND - tells sql use indexes on views (required on standard edition)
SELECT * 
FROM dbo.SumOfAllChargesByMember WITH (NOEXPAND)
ORDER BY memberno

-- What about forceplan?
USE pubs
go

--SET FORCEPLAN OFF
SELECT a.au_lname + ',' + substring(a.au_fname, 1, 1) + '.'
		AS 'Author''s Name'
	, t.title
	, p.pub_name
FROM dbo.titleauthor AS ta 
	RIGHT OUTER JOIN authors AS a
		ON a.au_id = ta.au_id
	LEFT OUTER JOIN titles AS t 
		ON t.title_id = ta.title_id	
	LEFT OUTER JOIN publishers AS p 
		ON p.pub_id = t.pub_id	
--OPTION (FORCE ORDER)
--where type = 'business'
go
SELECT a.au_lname + ',' + substring(a.au_fname, 1, 1) + '.'
		AS 'Author''s Name'
	, t.title
	, p.pub_name
FROM dbo.titleauthor AS ta 
	inner join titles AS t 
		ON t.title_id = ta.title_id	
	RIGHT OUTER join authors AS a
		ON a.au_id = ta.au_id
	LEFT OUTER JOIN publishers AS p 
		ON p.pub_id = t.pub_id	
--where type = 'business'
go
SELECT a.au_lname + ',' + substring(a.au_fname, 1, 1) + '.'
		AS 'Author''s Name'
	, t.title
	, p.pub_name
FROM dbo.authors AS a
	LEFT OUTER join titleauthor AS ta 
		ON a.au_id = ta.au_id
	LEFT OUTER join titles AS t 
		ON t.title_id = ta.title_id	
	LEFT OUTER JOIN publishers AS p 
		ON p.pub_id = t.pub_id	
--where type = 'business'
go
SELECT a.au_lname + ',' + substring(a.au_fname, 1, 1) + '.'
		AS 'Author''s Name'
	, t.title
	, p.pub_name
FROM dbo.titles AS t 
	JOIN publishers AS p 
		ON p.pub_id = t.pub_id	
	inner join titleauthor AS ta 
		ON t.title_id = ta.title_id	
	RIGHT OUTER join authors AS a
		ON a.au_id = ta.au_id
--where type = 'business'
go
--NOT POSSIBLE TO WRITE INNER JOINS IN THIS ORDER
--titles -> authors -> titleauthor
--authors -> titles -> titleauthor

